
require "Cocos2d"
require "Cocos2dConstants"
require "CocoStudio"

-- global var
local stoneTag = 8
local scissorTag = 9
local clothTag = 10
-- Button
local stoneBtn
local scissorBtn
local clothBtn
-- Panel
local leftHandPanel
local rightHandPanel
-- animation hand
local leftHand
local rightHand
-- Role
local myRole
local friendRole

-- cclog
cclog = function(...)
    print(string.format(...))
end

-- error stack
function __G__TRACKBACK__(msg)
    cclog("----------------------------------------")
    cclog("LUA ERROR: " .. tostring(msg) .. "\n")
    cclog(debug.traceback())
    cclog("----------------------------------------")
    return msg
end

-- close this game
local function closeEvent(sender, eventType)
    if eventType == ccui.TouchEventType.ended then
      --myRole:unregisterLuaHandler()
      --friendRole:unregisterLuaHandler()
      cc.Director:getInstance():popScene()
    end
end

local function showResult(leftRight, showResult)
    if leftRight == 0 then
      
      leftHand:getAnimation():stop()
      leftHand:setVisible(false)
      
      local showHand = leftHandPanel:getChildByTag(showResult)
      showHand:setVisible(true)
        
    elseif leftRight == 1 then
      
      rightHand:getAnimation():stop()
      rightHand:setVisible(false)
      
      local showHand = rightHandPanel:getChildByTag(showResult)
      showHand:setVisible(true)
      
    end
end

local function restoreJanKenPunch()
    for var = stoneTag, clothTag, 1 do
        cclog(var)
        local showLHand = leftHandPanel:getChildByTag(var)
        showLHand:setVisible(false)
        local showRHand = rightHandPanel:getChildByTag(var)
        showRHand:setVisible(false)
    end
    
    if not leftHand:isVisible() then
      leftHand:getAnimation():playWithIndex(0)
      leftHand:setVisible(true)
    end
    
    if not rightHand:isVisible() then
      rightHand:getAnimation():playWithIndex(0)
      rightHand:setVisible(true)
    end
    
    stoneBtn:setVisible(true)
    scissorBtn:setVisible(true)
    clothBtn:setVisible(true)
end

local function gameLogic(userSelect)
    local robotSelect = math.random(stoneTag, clothTag)
    
    showResult(0, userSelect)
    showResult(1, robotSelect)
    
    if userSelect == robotSelect then
      myRole:luaActionWithCallback("jump", restoreJanKenPunch)
      friendRole:luaActionWithCallback("jump", restoreJanKenPunch)
      return
    end
    
    local youWin = false
    if userSelect == stoneTag and robotSelect == scissorTag then
      youWin = true
    end
    if userSelect == scissorTag and robotSelect == clothTag then
      youWin = true
    end
    if userSelect == clothTag and robotSelect == stoneTag then
      youWin = true
    end
    
    if youWin then
      myRole:luaActionWithCallback("five", restoreJanKenPunch)
    else
      friendRole:luaActionWithCallback("five", restoreJanKenPunch)
    end
end

local function gameEvent(sender, eventType)
    if eventType == ccui.TouchEventType.ended then
      
      cclog(sender:getTag())
      
      gameLogic(sender:getTag())
      
      stoneBtn:setVisible(false)
      scissorBtn:setVisible(false)
      clothBtn:setVisible(false)
    end
end

local function main()
  
    collectgarbage("collect")
    -- avoid memory leak 这是脚本回收参数，避免内存泄漏
    collectgarbage("setpause", 100)
    collectgarbage("setstepmul", 5000)
    
    require('mobdebug').start()

    --[[
    -- 追加资源的搜索顺序
    cc.FileUtils:getInstance():addSearchResolutionsOrder("src");
    cc.FileUtils:getInstance():addSearchResolutionsOrder("res");

    local schedulerID = 0
    --support debug 获取目标平台
    local targetPlatform = cc.Application:getInstance():getTargetPlatform()
    if (cc.PLATFORM_OS_IPHONE == targetPlatform) or (cc.PLATFORM_OS_IPAD == targetPlatform) or
       (cc.PLATFORM_OS_ANDROID == targetPlatform) or (cc.PLATFORM_OS_WINDOWS == targetPlatform) or
       (cc.PLATFORM_OS_MAC == targetPlatform) then
        cclog("result is ")
        --require('debugger')()
    end

    -- 类似c++的include，引入文件，会检查是否重复引入
    -- require "hello2"
    -- 调用外部函数，在hello2.lua中
    -- cclog("result is " .. myadd(1, 1))
  
    ---------------
    -- 获取可视区域
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    -- 可视原点坐标 OpenGL坐标系，左下角为原点
    local origin = cc.Director:getInstance():getVisibleOrigin()
    ]]
    
    --local layer = CocoStudio.Director:getInstance():getWinSize()

    local scene = cc.Scene:create()
    local layer = ccs.GUIReader:getInstance():widgetFromJsonFile("JanKenPunch.json")
    
    ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("JanKenHand0.png", 
      "JanKenHand0.plist", "JanKenHand.ExportJson")
    
    if layer then
      
      local v_size = cc.Director:getInstance():getVisibleSize()
      
      local diff = (layer:getSize().width - v_size.width)/2
      layer:setPosition(-diff, 0)
      --[[if diff>0 then
        layer:setSize(v_size)
      end]]
      
      scene:addChild(layer)
      
      -- close Button and event
      local closeBtn = layer:getChildByTag(11)
      closeBtn:setPosition(v_size.width+diff-75, v_size.height-75)
      closeBtn:addTouchEventListener(closeEvent)
      
      -- vs hands
      leftHandPanel = layer:getChildByTag(12)
      rightHandPanel = layer:getChildByTag(13)
      
      if leftHandPanel then
        
        leftHand = ccs.Armature:create("JanKenHand")
        leftHand:setTag(0)
        leftHandPanel:addChild(leftHand)
        leftHand:setScaleX(-1.0)
        leftHand:getAnimation():playWithIndex(0)
        
        -- show result
        local stoneShow = cc.Sprite:create("stone.png")
        stoneShow:setScaleX(-1.0)
        stoneShow:setTag(stoneTag)
        stoneShow:setVisible(false)
        local scissorShow = cc.Sprite:create("scissor.png")
        scissorShow:setScaleX(-1.0)
        scissorShow:setTag(scissorTag)
        scissorShow:setVisible(false)
        local clothShow = cc.Sprite:create("cloth.png")
        clothShow:setScaleX(-1.0)
        clothShow:setTag(clothTag)
        clothShow:setVisible(false)
        leftHandPanel:addChild(stoneShow)
        leftHandPanel:addChild(scissorShow)
        leftHandPanel:addChild(clothShow)
      end
      
      if rightHandPanel then
        
        rightHand = ccs.Armature:create("JanKenHand")
        rightHand:setTag(1)
        rightHandPanel:addChild(rightHand)
        rightHand:getAnimation():playWithIndex(0)
        
        -- show result
        local stoneShow = cc.Sprite:create("stone.png")
        stoneShow:setTag(stoneTag)
        stoneShow:setVisible(false)
        local scissorShow = cc.Sprite:create("scissor.png")
        scissorShow:setTag(scissorTag)
        scissorShow:setVisible(false)
        local clothShow = cc.Sprite:create("cloth.png")
        clothShow:setTag(clothTag)
        clothShow:setVisible(false)
        rightHandPanel:addChild(stoneShow)
        rightHandPanel:addChild(scissorShow)
        rightHandPanel:addChild(clothShow)
      end
      
      -- vs character
      local leftCharacterPanel = layer:getChildByTag(6)
      
      if leftCharacterPanel then
        myRole = hr.CharacterManager:getInstance():getMyself("janken")
        myRole:setDirection(true, true)
        --myRole:registerLuaHandler(restoreJanKenPunch)
        leftCharacterPanel:addChild(myRole)
      end
      
      local rightCharacterPanel = layer:getChildByTag(7)
      
      if rightCharacterPanel then
        local friendID = hr.FriendManager:getInstance():getRoomOwnerID()
        friendRole = hr.CharacterManager:getInstance():getCharacterByID(friendID, "janken")
        friendRole:setDirection(false, true)
        --friendRole:registerLuaHandler(restoreJanKenPunch)
        rightCharacterPanel:addChild(friendRole)
      end
      
      -- game buttons
      stoneBtn = layer:getChildByTag(stoneTag)
      stoneBtn:addTouchEventListener(gameEvent)
      
      scissorBtn = layer:getChildByTag(scissorTag)
      scissorBtn:addTouchEventListener(gameEvent)
      
      clothBtn = layer:getChildByTag(clothTag)
      clothBtn:addTouchEventListener(gameEvent)
      
      -- change scene
      cc.Director:getInstance():pushScene(scene)
      
    end

end



--[[
xpcall( 调用函数, 错误捕获函数 );
lua提供了xpcall来捕获异常
xpcall接受两个参数:调用函数、错误处理函数。
当错误发生时,Lua会在栈释放以前调用错误处理函数,因此可以使用debug库收集错误相关信息。
两个常用的debug处理函数:debug.debug和debug.traceback
前者给出Lua的提示符,你可以自己动手察看错误发生时的情况;
后者通过traceback创建更多的错误信息,也是控制台解释器用来构建错误信息的函数。
--]]
local status, msg = xpcall(main, __G__TRACKBACK__)
if not status then
    error(msg)
end

